// EQPacket.cpp

#include "EQServerSocket_stdafx.h"

#include "EQPacket.h"
#include "UDPServerSocket.h"
#include "everquest.h"

CDatagramPacket CEQPacket::m_acker(10);


bool CEQPacket::DisAssemblePacket()
{
	unsigned int	len = m_data.getDataSize();
	unsigned char*	pBuffer = (unsigned char*)m_data.getData();

	// header & packet counter and checksum
	if(len < 8)
		return false;

	/* Check if the packet data is corrupted */
	if(GetDWord(pBuffer, len - 4) != CUDPServerSocket::getChk().getChecksum(pBuffer, len - 4, 0))
		return false;


	/* extract all aviable info from the packet */
	
	/* Packet header */
	unsigned short Header = GetWord(pBuffer, 0);
	unsigned int HeaderOffset = 4; /* Header + PacketCount are ALWAYS there ..*/


	/* Packet ID */
	m_PacketID = GetWord(pBuffer, 2);

	/*todo add removing of socket */
	/* FIN */
	if (Header & EQ_MSG_FIN)
		m_Finish = true;
	else
		m_Finish = false;

	/* ACK */
	if (Header & EQ_MSG_ACK)
	{
		/* Filter out the SEQ this packet is acknowledging */
		m_AcknowledgeID = GetWord(pBuffer, HeaderOffset);

		/* 2 bytes for ACK - ID */
		HeaderOffset += 2;
		
		m_Acknowledge = true;
	}else
	{
		m_AcknowledgeID = 0x0000;
		m_Acknowledge = false;
	}

	/* SEQ */
	if (Header & EQ_MSG_SEQ)
	{
		/* Filter out the SEQ of this packet */
		m_SequenceID = GetWord(pBuffer, HeaderOffset);

		/* 2 bytes for the SEQ - ID */
		HeaderOffset += 2;

		m_Sequenced = true;
	}else
	{
		m_SequenceID = 0x0000;
		m_Sequenced = false;
	}

	/* SYN */
	if (Header & EQ_MSG_SYN)
	{
		if (m_Sequenced == false)
			return false; /* There CANT be a SYN packet without SEQ flag set! */
		else
			m_Syncronize = true;
	}else
	{
		m_Syncronize = false;
	}

	/* this packet is not really known, but it has one byte after the SEQ (never seen any high level info with this packet.. but maybe the byte could even be after MaxFrag) */
	if (Header & EQ_MSG_UNK)
	{
		m_MSG_UNK = true;

		HeaderOffset += 1;
	}else
	{
		m_MSG_UNK = false;
	}

	/* FRG */
	if (Header & EQ_MSG_FRG)
	{
		m_Fragmented = true;

		/* get the frag id */
		m_FragID = GetWord(pBuffer, HeaderOffset);
		HeaderOffset += 2;
		
		/* get the frag index*/
		m_FragIndex = GetWord(pBuffer, HeaderOffset);
		HeaderOffset += 2; // frag index
		
		/* get the max frag index */
		m_MaxFragIndex = GetWord(pBuffer, HeaderOffset);
		HeaderOffset += 2; // max frag index
	}else
	{
		m_FragID = 0x0000;
		m_FragIndex = 0x0000;
		m_MaxFragIndex = 0x0000;
		m_Fragmented = false;
	}

	/* SYS */
	if (Header & EQ_MSG_SYS)
		m_System = true;
	else
		m_System = false;

	/* MSG */
	if (Header & EQ_MSG_MSG)
	{
		/* skip the 0x01 static byte */
		HeaderOffset += 1;

		m_Message = true;

		if (m_Sequenced == true)
		{
			m_MessageID = pBuffer[HeaderOffset];
			HeaderOffset += 1;
		}else
		{
			m_MessageID = 0x00;
		}

		/* assign data pointer. */
		m_Length = len - 4 /* Checksum */ - HeaderOffset;
		m_pData = (unsigned char *)&pBuffer[HeaderOffset];
	}else
	{
		if (m_Fragmented == true)
		{
			/* all fragments after the first fragment dont have MSG but have data */
			m_Length = len - 4 /* Checksum */ - HeaderOffset;
			m_pData = (unsigned char *)&pBuffer[HeaderOffset];
		}else
		{
			m_Message = false;
			m_MessageID = 0x00;

			m_pData = NULL;
			m_Length = 0;
		}
	}

	return true;
}

// Creates a packet to ACK this one.
CDatagramPacket *CEQPacket::getACKPacket(UWORD packetID)
{
	CBinary &dataObj = m_acker.getData();
	dataObj.setDataSize(10);
	unsigned char *pData = (unsigned char *)dataObj.getData();

	// set dest address
	m_acker.setInetAddr(this->getInetAddr());

	// Set the Header to ACK...
	FillWord(pData, 0, EQ_MSG_ACK);

	// Set packet ID
	FillWord(pData, 2, packetID);

	// Set the ID of the packet we're ACKing.
	FillWord(pData, 4, m_PacketID);

	// Set checksum.
	FillDWord(pData, 6, CUDPServerSocket::getChk().getChecksum(pData, 6, 0));

	return &m_acker;
}
