// EQServerSocketInterface.h

#ifndef __EQServerSocketInterface_h__
#define __EQServerSocketInterface_h__

#include "Binary.h"
#include "Object.h"
#include "StringClass.h"

#define EQ_MSG_NON 0x0000
#define EQ_MSG_SYS 0x0001
#define EQ_MSG_ACK 0x0004
#define EQ_MSG_UNK 0x0010		// unknown, 1 byte after SEQ .. means one missed packet (the one thats after the ACK'ed in this packet .. so if its acked for 0x0001 .. the missng packet is 0x0002
#define EQ_MSG_SEQ 0x0200
#define EQ_MSG_CLS 0x0400		// server closing connection
#define EQ_MSG_FRG 0x0800
#define EQ_MSG_MSG 0x1000
#define EQ_MSG_SYN 0x2000
#define EQ_MSG_FIN 0x4400		// client finished


class IEQSocketException
{
public:
	virtual ~IEQSocketException()
	{
	}

	virtual String GetDescription() = 0;
};

class IClientConnection : public CObject
{
public:
	// returns true if there are packets waiting inthe recieve queue.
	virtual bool IsPacketQueued() = 0;

	// retrieves packet from recive queue and removes it too.
	virtual CBinary *GetRecvPacket() = 0;

	// Called to send a message to the client.
	virtual void Send(CBinary *txMsg, unsigned short headers = 0) = 0;

	bool operator==(const IClientConnection &o)
	{
		return this == &o;
	}
};

class ISocketListner : public CObject
{
public:
	// Called whenver a new client connects.
	virtual void NewClient(IClientConnection *client) = 0;

	// Called whenever a client disconnects.
	virtual void ClientRemoved(IClientConnection *client) = 0;

	// Called each time a new packet is recieved.
	virtual void Recieve(IClientConnection *client, CBinary *msg) = 0;
};

class IServerSocket : public CObject
{
protected:
	ISocketListner *m_listener;

public:
	IServerSocket(ISocketListner *listener = NULL)
		: m_listener(listener)
	{}

	virtual ~IServerSocket()
	{}

	virtual void bind(unsigned short localPort, const char* localAddressName = NULL) = 0;

	virtual void startLogging(const char* logFileName) = 0;
	virtual void stopLogging() = 0;

	// Tests if any new clients have connected. Returns the pointer to the
	// ClientConnection object that represents the new client if true, 
	// otherwise, returns NULL.
	virtual IClientConnection *IsNewClient() = 0;

	// Tests if any clients have disconnected. Returns the pointer to the
	// ClientConnection object that represents the client that disconnected
	// if true, otherwise, returns NULL.
	virtual IClientConnection *IsRemovedClient() = 0;

	// Removes the specified CClientConnection object from the m_clients
	// vector and adds it to the m_ClosedClientConnections vector.
	virtual void RemoveClientSocket(IClientConnection *client) = 0;

	// deletes the client and removes from m_ClosedClientConnections.
	virtual void FinalizeClient(IClientConnection *client) = 0;

	// Only one listner allowed.
	void SetListener(ISocketListner *listener)
	{
		m_listener = listener;
	}
};


#endif // __EQServerSocketInterface_h__